#!/usr/bin/env python

import os
import sys
import tarfile
import csv
import ConfigParser
import shutil
import tempfile
import logging
import datetime

OMERO_PYTHON = "/opt/Omero/lib/python"                   #  without these, (which set the PYTHONPATH 
sys.path.append(OMERO_PYTHON)                            #  environment variable), the following imports
                                                         #  will fail!
import omero                      
import omero.cli
import omero.gateway
import omero.rtypes 
import omero.model



"""  Global environment variables  """
config = ConfigParser.ConfigParser()
config.readfp(open(r'/root/omero-upload/upload.config'))
logFile = "/root/omero-upload/upload.log"
logging.basicConfig(filename=logFile,level=logging.INFO)
user = config.get('upload', 'user')
passwd = config.get('upload', 'passwd')
port = config.get('upload', 'port')
projId = config.get('upload', 'projId')
imageTypes = config.get('upload', 'imageTypes')
server = "localhost"
self = os.path.basename(__file__)
imageTypes = imageTypes.split(',')



def connect():
    """  Function to connected to BlitzGateway  """
    conn = omero.gateway.BlitzGateway(user, passwd, host=server, port=port)
    connected = conn.connect()
    if not connected:
        logging.info('%s: Error - Unable to connect to Omero - exiting' % (self))
        sys.exit(5)
    logging.info('%s: Connection to Omero made' % (self))
    return conn



def addProject(conn, project_name):
    """  Make new OMERO project in current group, returning the new project Id.  """
    proj = omero.model.ProjectI()
    proj.name = omero.rtypes.rstring(str(project_name))
    upServ = conn.getUpdateService()
    ret = upServ.saveAndReturnObject(proj, conn.SERVICE_OPTS)
    ret.unload()
    logging.info('%s: Project added: %s' % (self, ret.id.val))
    return ret.id.val



def addDataSet(conn, project_id, dataset_name):
    """   Make new OMERO dataset, returning the new dataset Id.  """
    dset = omero.model.DatasetI()
    dset.name = omero.rtypes.rstring(str(dataset_name))
    proj_dset = omero.model.ProjectDatasetLinkI()
    proj = conn.getObject("Project", project_id)
    proj_dset.setParent(proj._obj)
    proj_dset.setChild(dset)
    dset.addProjectDatasetLink(proj_dset)
    upServ = conn.getUpdateService()
    ret = upServ.saveAndReturnObject(dset, conn.SERVICE_OPTS)
    ret.unload()
    logging.info('%s: Dataset added: %s' % (self, ret.id.val))
    return ret.id.val



def callOmeroCLI(cli, import_args):
    """  Function to try the call to the Omero command line interface.
         If the call falls, try again - this call sometimes failes with 
         a "NonZeroReturnCode", but when I try again it works! 
         Therefore try three times and then exit with an error! """

    try:
        cli.invoke(import_args, strict=True)

    except omero.cli.NonZeroReturnCode, err:
        logging.warning('%s: Omero CLI has had a NonZeroReturnCode %s, trying again' % (self, err))

        try:
            cli.invoke(import_args, strict=True)

        except omero.cli.NonZeroReturnCode, err:
            logging.warning('%s: Omero CLI has had a NonZeroReturnCode %s, trying again' % (self, err))

            try:
                cli.invoke(import_args, strict=True)

            except:
                logging.error('%s: Omero CLI has failed three times - exiting.' % (self))
                sys.exit(7)



def addImage(conn, dset_id, filePath):
    """  Import filename using OMERO CLI.  Return : OMERO image Id  """
    dataset = dset_id
    cli = omero.cli.CLI()
    cli.loadplugins()
    import_args = ["import"]
    import_args.extend(["-s", str(server)])
    import_args.extend(["-u", str(user)])
    import_args.extend(["-w", str(passwd)])
    import_args.extend(["-p", str(port)])
    import_args.extend(["-d", str(dataset)])
    name = os.path.basename(filePath)
    import_args.extend(["-n", str(name)])
    clie = "cli.err"
    clio = "cli.out"
    import_args.extend(["---errs=" + clie, "---file=" + clio, "--"])
    import_args.append(filePath)
    logging.info('%s: About to write image with following arguments: %s' % (self, import_args))
    
    callOmeroCLI(cli, import_args)

    pix_id = int(open(clio, 'r').read().rstrip())
    im_id = conn.getQueryService().get("Pixels", pix_id).image.id.val
    logging.info('%s: Image added: %s' % (self, im_id))
    return im_id



def attach(conn, im_id, attachment):
    """  Function to add an attachment!  """
    img = conn.getObject("Image", im_id)
    fann = conn.createFileAnnfromLocalFile(attachment)
    img.linkAnnotation(fann)
    img.save()
    logging.info('%s: Added attachment to : %s' % (self, im_id))



def addTag(conn, id, text):
    """  Function to add a tag!  """
    tag = omero.model.TagAnnotationI()
    tag.textValue = omero.rtypes.wrap(str(text))
    upServ = conn.getUpdateService()
    annotation = upServ.saveAndReturnObject(tag, conn.SERVICE_OPTS)
    link = omero.model.DatasetAnnotationLinkI()
    link.setParent(omero.model.DatasetI(id, False))
    link.setChild(annotation)
    upServ.saveAndReturnObject(link)
    logging.info('%s: Added Tag to : %s' % (self, id))
    return annotation



def linkTag(conn, id, annotation, im_ind):
    """  Function to LINK a tag! """
    upServ = conn.getUpdateService()
    if im_ind:
        link = omero.model.ImageAnnotationLinkI()
        link.setParent(omero.model.ImageI(id, False))
    else:
        link = omero.model.DatasetAnnotationLinkI()
        link.setParent(omero.model.DatasetI(id, False))
        
    link.setChild(annotation)
    logging.info('%s: Linker Tag to : %s' % (self, id))
    upServ.saveAndReturnObject(link)



def unTar(infile):
    """  Function to Untar the file  """

    try:
        tarf = tarfile.open(infile, "r")
 
    except:
        logging.error('%s: Error opening TAR file - exiting' % (self))
        sys.exit(6)

    logging.info('%s: Opened TAR file. All OK' % (self))

    folder = tempfile.mkdtemp()

    for item in tarf.getnames():
        tarf.extract(item, path=folder)

    tarf.close()
    logging.info('%s: unpacked file' % (self))
    return folder



def connectAndAdd(Blitz, line, folder, issueInd):
    """  Function to connect to Omero and add the images, attachments and tags!  """
    rootname = line[4]
    rootname = line[4]
    anchor = line[6]
    tags = line[:3]
    anchorPath = os.path.join(folder, rootname, anchor)

    anchorID = 0

    tmpPath = os.path.join(folder, rootname)
    if not os.path.exists(tmpPath) or not os.listdir(tmpPath) or not rootname:
        logging.error('%s: No files exist for %s - exiting' % (self, rootname))
        issueInd = True                                  #  Check that the files exist
                                                         #  to upload!
    else: 
        for tag in line[7].split(';'):
            tags.append(tag)                             #  Compile all tags into a list

        tags = set(tags)                                 #  Remove any duplicate tags 
                                                         #  (a set cannot have duplicates)
        dsetId = addDataSet(Blitz, projId, rootname)     #  Create the dataset
                                                         #  will fail!

        if anchor and os.path.exists(anchorPath):  
            anchorId = addImage(Blitz, dsetId, anchorPath)   #  Add anchor image first.
        else:
            anchorId = 0
            logging.info('%s: Anchor image not found for folder : %s' % (self, rootname))

        for filename in os.listdir(os.path.join(folder, rootname)):
            if not filename == anchor:                   #  Anchor image already added
                filenamePath = os.path.join(folder, rootname, filename)
                imageId = 0
                for element in imageTypes:
                    element = "." + element
                    if filename.endswith((element)):     #  This is an image, add it
                        imageId = addImage(Blitz, dsetId, filenamePath)
    
                if (not imageId) and anchorId:           #  Not an image, attach it
                    attach(Blitz, anchorId, filenamePath)
    
        for tag in tags:
            obs = Blitz.getObjects("TagAnnotation", attributes={'textValue':tag})
            obs = list(obs)
    
            if obs:
                linkTag(Blitz, dsetId, obs[0]._obj, False)
                if anchorId:
                    linkTag(Blitz, anchorId, obs[0]._obj, True)

            else:
                annotation = addTag(Blitz, dsetId, tag)
                if anchorId:
                    linkTag(Blitz, anchorId, annotation, True)

    return issueInd
    


def main(infile):
    """  Main section of the script.  """
    outDate = datetime.datetime.now()
    outDate = outDate.strftime("%c")
    logging.info('%s: Starting - %s ' % (self, outDate))
    global projId

    if not os.path.isfile(infile):
        logging.error('%s: Incorrect Usage.  Input file does not exist - exiting' % (self))
        sys.exit(2)
			
    folder = unTar(infile)                               #  unpack the file into tmp
    csvPath = os.path.join(folder, "input.file.txt")

    try:
        csvFile = open(csvPath, "rbU")

    except IOError, err:
        logging.error('%s: CSV file does not exist - this is required: %s' % (self, csvPath))
        sys.exit(3)
    
    csvList = csv.reader(csvFile)

    Blitz = connect()                                    #  Connect to blitz

    issueInd = False
    for csvLine in csvList:
        if not csvLine[0] == "Name":                     #  Ignore the header line 
            issueInd = connectAndAdd(Blitz, csvLine, folder, issueInd)

    shutil.rmtree(folder)
    outDate = datetime.datetime.now()
    outDate = outDate.strftime("%c")

    if issueInd:
        logging.info('%s: Completed - but with issues %s ' % (self, outDate))
        return issueInd
    else:
        logging.info('%s: Completed - %s ' % (self, outDate))
        return 0



if __name__ == "__main__":
    try:  
        infile = sys.argv[1]

    except IndexError, err:
        logging.error('%s: Error - input file is required - exiting' % (self))
        sys.exit(1)

    sys.exit(main(sys.argv[1]))
