#!/usr/bin/env python

import os 
import sys
import ConfigParser
import glob
import shutil
import logging
import smtplib
import tarfile


 
"""  Global environment variables  """
config = ConfigParser.ConfigParser()
config.readfp(open(r'/root/omero-upload/upload.config'))
logFile = "/var/log/omero-upload.log"
logging.basicConfig(filename=logFile,level=logging.INFO)
importDir = config.get('upload', 'importDir')
scpTarget = config.get('upload', 'scpTarget')
emailAddr = config.get('upload', 'emailAddr')
userAddr  = config.get('upload', 'userAddr')
completedDir = os.path.join(importDir, 'Completed')
errorDir = os.path.join(importDir, 'Error')
transDir = os.path.join(importDir, 'Transferred')
scpDir = os.path.join(importDir, 'External')
self = os.path.basename(__file__)
hostname = os.uname()[1]


""" Set up of the email, which will be sent in the event of an error """
header = """From: %s.%s@nbi.ac.uk \n 
To: %s 
Subject: Omero Bulk Upload \n""" % (self, hostname, emailAddr)
msgBody1 = "\nAn error has occured with the Omero bulk upload script %s. " % (self)
msgBody2 = "\n\nThe error occured while attempting to upload the following file(s):\n\t"
msgBody3 = """\n\nIt was attempting to uploaded files on the Omero server:\n\t%s.
The logfile on this server is:\n\t%s.  
The source location for the ZIP files is:\n\t%s.""" % (hostname, logFile, importDir)
msgBody4 = "\n\nA lockfile has been found, but no omero-upload process is currently running."
msgBody5 = """\n\nInformation specified in the Input file was not found.  
Not all files have therefore been uploaded successfully.  """



def createLock():
    """  Function to check for lockfile, abort if found, otherwise create lock!  """
    
    lockFile = os.path.join("/tmp/", self)
    globFile = lockFile + ".*"

    lock = glob.glob(globFile)
    if lock:
        PID = lock[0].split('.')[-1]
        if os.path.exists(os.path.join('/proc',PID)):
            logging.error('%s: Pid is %s ' % (self, PID))
            logging.error('%s: Lockfile found - previous instance running.  Exiting' % (self))
            sys.exit(1)

        else:
            logging.error('%s: Lockfile(s) %s found,  but no process identified' % (self, lock))
            receiver = [emailAddr]
            sender = self + ".Script@" + hostname
            message = header + msgBody1 + msgBody4
            message = message + msgBody3
            smtpObj = smtplib.SMTP('localhost')
            smtpObj.sendmail(sender, receiver, message)

    PID = os.getpid()
    lockFile = lockFile + "." + str(PID)
    open(lockFile, 'a').close()
    return lockFile



def checkTarFile(uploadFilePath, uploadFile):
    cmd = ''.join(["tar -tvf ", uploadFilePath, " > /dev/null 2>&1"])
    if tarfile.is_tarfile(uploadFilePath) and not os.system(cmd):
        return True
    else:
        logging.info('%s: File not a valid TAR file: %s ' % (self, uploadFilePath))
        return False



def serverSideUpload(uploadFiles):
    """  This function loops through the ZIP files and uploads each one
         using the ServerSideUpload code (seperate module) """
    global message
    import ServerSideUpload
    toUpload = []
    errorCount = 0
    emailUser = False
    for uploadFile in uploadFiles:
        uploadFilePath = os.path.join(importDir, uploadFile)
        if (not uploadFile.startswith('.') 
            and os.path.isfile(uploadFilePath)
            and checkTarFile(uploadFilePath, uploadFile) ):

            toUpload.append(uploadFile)
            
            try:
                retErr = ServerSideUpload.main(uploadFilePath)
                shutil.move(uploadFilePath, completedDir)
                if retErr:
                    emailUser = True
                    errorCount += 1

            except SystemExit:
                logging.error('%s: System exit attempting to upload file %s ' % (self, uploadFilePath))
                logging.error('%s: Moving %s to the Error directory ' % (self, uploadFilePath))
                shutil.move(uploadFilePath, errorDir)
                errorCount += 1


    if emailAddr and toUpload and errorCount != 0:
        receiver = [emailAddr]
        sender = self + ".Script@" + hostname
        message = header + msgBody1 + msgBody2 + str(toUpload)
        message = message + msgBody3
        smtpObj = smtplib.SMTP('localhost')
        smtpObj.sendmail(sender, receiver, message)

    if emailUser:
        receiver = [userAddr]
        sender = self + ".Script@" + hostname
        message = header + msgBody1 + msgBody2 + str(toUpload)
        message = message + msgBody5
        smtpObj = smtplib.SMTP('localhost')
        smtpObj.sendmail(sender, receiver, message)



def main():
    """  Main code states here  """
    lockFile = createLock()

    if not os.path.exists(importDir):
        logging.error('%s: Import directory does not exist: %s' % (self, importDir))
        sys.exit(1)

    uploadFiles = os.listdir(importDir)
    if uploadFiles:
        serverSideUpload(uploadFiles)

    if scpTarget:
        scpFiles = os.listdir(scpDir)
        if scpFiles:
            for scpFile in scpFiles:
                scpFilePath = os.path.join(scpDir, scpFile)
                os.system('scp "%s" "%s"' % (scpFilePath, scpTarget))
                shutil.move(scpFilePath, transDir)
 
    os.remove(lockFile)
 


if __name__ == "__main__":
    main()
